@@ -3,54 +3,13 @@
 
 #define ADNN_MM_TRANSPOSE 1
 
-#include <cmath>
-#include <cassert>
-#include <algorithm>
 #include "dropout_gpu_emulator.hpp"
 
-int sumvc(std::vector<int>& x)
-{
-    int sum = 0;
-    for(int i = 0; i < x.size(); i++)
-    {
-        sum += x[i];
-    }
-    return sum;
-}
+#include <../test/rnn_util.hpp>
 
-template <typename T>
-T activfunc(T x, int actvf)
-{
-    T alpha = static_cast<T>(1), beta0 = static_cast<T>(0), beta1 = static_cast<T>(1);
-    if(actvf == 0)
-    {
-        //        float y = 0;
-        //        return std::max(x, y);
-        return (x > 0) ? x : x * beta0;
-    }
-    else if(actvf == 2)
-    {
-        return 1 / (1 + exp(-x));
-    }
-
-    //    return tanh(x);
-    return alpha * tanh(beta1 * x);
-}
-
-template <typename T>
-T dervactivfunc(T x, int actvf)
-{
-    if(actvf == 0)
-    {
-        return (x > 0 ? 1 : 0);
-    }
-    else if(actvf == 2)
-    {
-        return exp(-x) / (1 + exp(-x)) / (1 + exp(-x));
-    }
-
-    return 1 / cosh(x) / cosh(x);
-}
+#include <algorithm>
+#include <cassert>
+#include <cmath>
 
 template <typename Tgpu, typename Tref>
 void RunRNNForwardGEMMCPUVerify(miopenHandle_t handle,
@@ -148,7 +107,7 @@ void RunRNNForwardGEMMCPUVerify(miopenHandle_t handle,
     int wei_shift_bias = ((in_h + hy_h) * bi + (bi * hy_h + hy_h) * bi * (numlayer - 1)) * hy_h;
 
     // initial dropoput
-    std::vector<prngStates> dropout_states_host;
+    std::vector<rocrand_state_xorwow> dropout_states_host;
     std::vector<unsigned char> dropout_reservespace_host;
     std::vector<Tref> dropout_hid_state;
     miopenTensorDescriptor_t dropout_inputTensor{}, dropout_outputTensor{};
@@ -156,8 +115,8 @@ void RunRNNForwardGEMMCPUVerify(miopenHandle_t handle,
     {
         size_t statesSizeInBytes = 0;
         miopenDropoutGetStatesSize(handle, &statesSizeInBytes);
-        size_t states_size  = statesSizeInBytes / sizeof(prngStates);
-        dropout_states_host = std::vector<prngStates>(states_size);
+        size_t states_size  = statesSizeInBytes / sizeof(rocrand_state_xorwow);
+        dropout_states_host = std::vector<rocrand_state_xorwow>(states_size);
         InitKernelStateEmulator(dropout_states_host, dropoutDesc);
 
         std::array<int, 2> drop_in_len  = {{batch_n, hy_h * bi}};
