@@ -1,10 +1,11 @@
+#include <miopen/buffer_info.hpp>
 #include <miopen/conv/invokers/impl_gemm_dynamic.hpp>
 #include <miopen/conv/data_invoke_params.hpp>
 #include <miopen/algorithm.hpp>
 #include <miopen/handle.hpp>
 #include <miopen/tensor_ops.hpp>
 #include <miopen/solver/implicitgemm_util.hpp>
-#include <miopen/util_sol.hpp>
+#include <miopen/batched_transpose_sol.hpp>
 #include <boost/any.hpp>
 
 namespace miopen {
@@ -32,7 +33,7 @@ static inline uint32_t igemm_find_tile_size_with_upper_bound(
 }
 
 static float CallImplGemmDynamicForward1x1(const miopen::Handle& handle,
-                                           const ProblemDescription& conv_problem,
+                                           const ProblemDescription& problem,
                                            ConstData_t src,
                                            Data_t dst,
                                            ConstData_t wei,
@@ -44,19 +45,19 @@ static float CallImplGemmDynamicForward1x1(const miopen::Handle& handle,
     MIOPEN_LOG_I(kernel.GetName());
 
     // clang-format off
-    int hi          = conv_problem.GetInHeight();
-    int wi          = conv_problem.GetInWidth();
-    int n           = conv_problem.GetInBatchSize();
-    int k           = conv_problem.GetOutChannels();
-    int c           = conv_problem.GetInChannels();
-    int ho          = conv_problem.GetOutHeight();
-    int wo          = conv_problem.GetOutWidth();
-    int stride_h    = conv_problem.GetKernelStrideH();
-    int stride_w    = conv_problem.GetKernelStrideW();
-    int dilation_h  = conv_problem.GetDilationH();
-    int dilation_w  = conv_problem.GetDilationW();
-    int pad_h       = conv_problem.GetPadH();
-    int pad_w       = conv_problem.GetPadW();
+    int hi          = problem.GetInHeight();
+    int wi          = problem.GetInWidth();
+    int n           = problem.GetInBatchSize();
+    int k           = problem.GetOutChannels();
+    int c           = problem.GetInChannels();
+    int ho          = problem.GetOutHeight();
+    int wo          = problem.GetOutWidth();
+    int stride_h    = problem.GetKernelStrideH();
+    int stride_w    = problem.GetKernelStrideW();
+    int dilation_h  = problem.GetDilationH();
+    int dilation_w  = problem.GetDilationW();
+    int pad_h       = problem.GetPadH();
+    int pad_w       = problem.GetPadW();
     int gap_0     = 0;
     // clang-format on
 
@@ -86,11 +87,9 @@ static float CallImplGemmDynamicForward1x1(const miopen::Handle& handle,
     return elapsed;
 }
 
-InvokerFactory
-MakeImplGemmDynamicForward1x1InvokerFactory(const miopen::ProblemDescription& problem)
+InvokerFactory MakeImplGemmDynamicForward1x1InvokerFactory(const ProblemDescription& problem)
 {
-    const auto& conv_problem = problem.conv_problem;
-    return [conv_problem](const std::vector<Kernel>& kernels) {
+    return [problem](const std::vector<Kernel>& kernels) {
         return [=](const Handle& handle, const AnyInvokeParams& primitive_parameters) {
             decltype(auto) data_ctx = primitive_parameters.CastTo<conv::DataInvokeParams>();
             const auto& tensors     = data_ctx.tensors;
@@ -103,7 +102,7 @@ MakeImplGemmDynamicForward1x1InvokerFactory(const miopen::ProblemDescription& pr
                            [&](const Kernel& k) { return handle.Run(k); });
             float elapsed = 0;
             elapsed       = CallImplGemmDynamicForward1x1(
-                handle, conv_problem, tensors.in, tensors.out, tensors.w, ks);
+                handle, problem, tensors.in, tensors.out, tensors.w, ks);
             if(handle.IsProfilingEnabled())
             {
                 handle.ResetKernelTime();
@@ -113,27 +112,24 @@ MakeImplGemmDynamicForward1x1InvokerFactory(const miopen::ProblemDescription& pr
     };
 }
 
-template <>
-InvokerFactory
-MakeImplGemmDynamicBackwardDataInvokerFactory<int>(const miopen::ProblemDescription& problem,
-                                                   const int& cfg)
+InvokerFactory MakeImplGemmDynamicBackwardDataInvokerFactory(const ProblemDescription& problem,
+                                                             const int cfg)
 {
-    const auto& conv_problem = problem.conv_problem;
-    int hi                   = conv_problem.GetOutHeight();
-    int wi                   = conv_problem.GetOutWidth();
-    int n                    = conv_problem.GetInBatchSize();
-    int k                    = conv_problem.GetInChannels();
-    int c                    = conv_problem.GetOutChannels();
-    int ho                   = conv_problem.GetInHeight();
-    int wo                   = conv_problem.GetInWidth();
-    int stride_h             = conv_problem.GetInHeight() > 1 ? conv_problem.GetKernelStrideH() : 1;
-    int stride_w             = conv_problem.GetInWidth() > 1 ? conv_problem.GetKernelStrideW() : 1;
-    int dilation_h = conv_problem.GetWeightsHeight() > 1 ? conv_problem.GetDilationH() : 1;
-    int dilation_w = conv_problem.GetWeightsWidth() > 1 ? conv_problem.GetDilationW() : 1;
-    int pad_h      = conv_problem.GetPadH();
-    int pad_w      = conv_problem.GetPadW();
-    int y          = conv_problem.GetWeightsHeight();
-    int x          = conv_problem.GetWeightsWidth();
+    int hi         = problem.GetOutHeight();
+    int wi         = problem.GetOutWidth();
+    int n          = problem.GetInBatchSize();
+    int k          = problem.GetInChannels();
+    int c          = problem.GetOutChannels();
+    int ho         = problem.GetInHeight();
+    int wo         = problem.GetInWidth();
+    int stride_h   = problem.GetInHeight() > 1 ? problem.GetKernelStrideH() : 1;
+    int stride_w   = problem.GetInWidth() > 1 ? problem.GetKernelStrideW() : 1;
+    int dilation_h = problem.GetWeightsHeight() > 1 ? problem.GetDilationH() : 1;
+    int dilation_w = problem.GetWeightsWidth() > 1 ? problem.GetDilationW() : 1;
+    int pad_h      = problem.GetPadH();
+    int pad_w      = problem.GetPadW();
+    int y          = problem.GetWeightsHeight();
+    int x          = problem.GetWeightsWidth();
 
     int gcd_stride_dilation_h = solver::gcd(stride_h, dilation_h);
     int gcd_stride_dilation_w = solver::gcd(stride_w, dilation_w);
@@ -182,9 +178,7 @@ MakeImplGemmDynamicBackwardDataInvokerFactory<int>(const miopen::ProblemDescript
         const int gemm_k_gid = k * y_dot_slice_gid[gemm_id] * x_dot_slice_gid[gemm_id];
         is_gemm_not_empty.emplace_back(gemm_k_gid > 0);
     }
-    bool need_set_zero = false;
-    if(y < stride_h || x < stride_w || dilation_h != 1 || dilation_w != 1)
-        need_set_zero = true;
+    bool need_set_zero = true;
 
     return [=](const std::vector<Kernel>& kernels) {
         const auto kernel = kernels[0];
@@ -251,28 +245,26 @@ MakeImplGemmDynamicBackwardDataInvokerFactory<int>(const miopen::ProblemDescript
     };
 }
 
-template <>
 InvokerFactory
-MakeImplGemmDynamicBackwardDataInvokerFactory<solver::TunableImplicitGemmGTCDynamic_t>(
-    const miopen::ProblemDescription& problem, const solver::TunableImplicitGemmGTCDynamic_t& cfg)
+MakeImplGemmDynamicBackwardDataInvokerFactory(const ProblemDescription& problem,
+                                              const solver::TunableImplicitGemmGTCDynamic_t& cfg)
 {
-    const auto& conv_problem = problem.conv_problem;
-    int hi                   = conv_problem.GetOutHeight();
-    int wi                   = conv_problem.GetOutWidth();
-    int n                    = conv_problem.GetInBatchSize();
-    int k                    = conv_problem.GetInChannels();
-    int c                    = conv_problem.GetOutChannels();
-    int ho                   = conv_problem.GetInHeight();
-    int wo                   = conv_problem.GetInWidth();
-    int stride_h   = conv_problem.GetOutHeight() > 1 ? conv_problem.GetKernelStrideH() : 1;
-    int stride_w   = conv_problem.GetOutWidth() > 1 ? conv_problem.GetKernelStrideW() : 1;
-    int dilation_h = conv_problem.GetWeightsHeight() > 1 ? conv_problem.GetDilationH() : 1;
-    int dilation_w = conv_problem.GetWeightsWidth() > 1 ? conv_problem.GetDilationW() : 1;
-    int pad_h      = conv_problem.GetPadH();
-    int pad_w      = conv_problem.GetPadW();
-    int y          = conv_problem.GetWeightsHeight();
-    int x          = conv_problem.GetWeightsWidth();
-    int group      = conv_problem.GetGroupCount();
+    int hi         = problem.GetOutHeight();
+    int wi         = problem.GetOutWidth();
+    int n          = problem.GetInBatchSize();
+    int k          = problem.GetInChannels();
+    int c          = problem.GetOutChannels();
+    int ho         = problem.GetInHeight();
+    int wo         = problem.GetInWidth();
+    int stride_h   = problem.GetOutHeight() > 1 ? problem.GetKernelStrideH() : 1;
+    int stride_w   = problem.GetOutWidth() > 1 ? problem.GetKernelStrideW() : 1;
+    int dilation_h = problem.GetWeightsHeight() > 1 ? problem.GetDilationH() : 1;
+    int dilation_w = problem.GetWeightsWidth() > 1 ? problem.GetDilationW() : 1;
+    int pad_h      = problem.GetPadH();
+    int pad_w      = problem.GetPadW();
+    int y          = problem.GetWeightsHeight();
+    int x          = problem.GetWeightsWidth();
+    int group      = problem.GetGroupCount();
 
     int gcd_stride_dilation_h = solver::gcd(stride_h, dilation_h);
     int gcd_stride_dilation_w = solver::gcd(stride_w, dilation_w);
@@ -321,13 +313,10 @@ MakeImplGemmDynamicBackwardDataInvokerFactory<solver::TunableImplicitGemmGTCDyna
         const int gemm_k_gid = k * y_dot_slice_gid[gemm_id] * x_dot_slice_gid[gemm_id];
         is_gemm_not_empty.emplace_back(gemm_k_gid > 0);
     }
-    bool need_set_zero = false;
-    if(y < stride_h || x < stride_w || dilation_h != 1 || dilation_w != 1)
-        need_set_zero = true;
-
-    int nxb = cfg.nxb;
-    int b   = h_tilda_slice * w_tilda_slice;
-    b       = (cfg.nxe == 0) ? (b) : ((b + nxb - 1) / nxb) * nxb; // pad to nxb modulo when nxe != 0
+    bool need_set_zero = true;
+    int nxb            = cfg.nxb;
+    int b              = h_tilda_slice * w_tilda_slice;
+    b = (cfg.nxe == 0) ? (b) : ((b + nxb - 1) / nxb) * nxb; // pad to nxb modulo when nxe != 0
 
     uint32_t nb_n0          = cfg.tensor_b_cluster_lengths[2] * cfg.tensor_b_thread_lengths[2];
     uint32_t nb_n1b         = cfg.tensor_b_cluster_lengths[3] * cfg.tensor_b_thread_lengths[3];
@@ -441,30 +430,29 @@ MakeImplGemmDynamicBackwardDataInvokerFactory<solver::TunableImplicitGemmGTCDyna
 }
 
 InvokerFactory MakeImplGemmDynamicForwardXdlopsNHWCInvokerFactory(
-    const ConvolutionContext& ctx,
-    const miopen::ProblemDescription& problem,
-    const solver::PerformanceConfigAsmImplicitGemmGTCFwdXdlopsNHWC& config)
+    const ExecutionContext& ctx,
+    const ProblemDescription& problem,
+    const solver::conv::PerformanceConfigAsmImplicitGemmGTCFwdXdlopsNHWC& config)
 {
-    const auto& conv_problem = problem.conv_problem;
-    int hi                   = conv_problem.GetInHeight();
-    int wi                   = conv_problem.GetInWidth();
-    int n                    = conv_problem.GetInBatchSize();
-    int k                    = conv_problem.GetOutChannels();
-    int c                    = conv_problem.GetInChannels();
-    int ho                   = conv_problem.GetOutHeight();
-    int wo                   = conv_problem.GetOutWidth();
-    int stride_h             = conv_problem.GetKernelStrideH();
-    int stride_w             = conv_problem.GetKernelStrideW();
-    int dilation_h           = conv_problem.GetDilationH();
-    int dilation_w           = conv_problem.GetDilationW();
-    int pad_h                = conv_problem.GetPadH();
-    int pad_w                = conv_problem.GetPadW();
-    int y                    = conv_problem.GetWeightsHeight();
-    int x                    = conv_problem.GetWeightsWidth();
-    int group                = conv_problem.GetGroupCount();
-    int c_karg               = c / group;
-    int y_karg               = y;
-    int x_karg               = x;
+    int hi         = problem.GetInHeight();
+    int wi         = problem.GetInWidth();
+    int n          = problem.GetInBatchSize();
+    int k          = problem.GetOutChannels();
+    int c          = problem.GetInChannels();
+    int ho         = problem.GetOutHeight();
+    int wo         = problem.GetOutWidth();
+    int stride_h   = problem.GetKernelStrideH();
+    int stride_w   = problem.GetKernelStrideW();
+    int dilation_h = problem.GetDilationH();
+    int dilation_w = problem.GetDilationW();
+    int pad_h      = problem.GetPadH();
+    int pad_w      = problem.GetPadW();
+    int y          = problem.GetWeightsHeight();
+    int x          = problem.GetWeightsWidth();
+    int group      = problem.GetGroupCount();
+    int c_karg     = c / group;
+    int y_karg     = y;
+    int x_karg     = x;
 
     int splits_4G = solver::igemm_split_batch_size(
         hi, wi, ho, wo, n, k, c, miopen::GetTypeSize(problem.GetInDataType()));
@@ -503,7 +491,8 @@ InvokerFactory MakeImplGemmDynamicForwardXdlopsNHWCInvokerFactory(
         shift_pack_1 = 0;
     }
 
-    bool need_set_zero                 = config.gemm_k_global_split > 0;
+    // Clear buffer for all condition to resolve the NaN issue.
+    bool need_set_zero                 = true;
     bool use_fp32_global_split_on_fp16 = config.vector_store == 1 && config.gemm_k_global_split > 0;
 
     std::vector<OpKernelArg> opArgs;
@@ -539,15 +528,15 @@ InvokerFactory MakeImplGemmDynamicForwardXdlopsNHWCInvokerFactory(
 
     std::vector<std::vector<OpKernelArg>> opArgsTrans;
 
-    const auto lowp_quant = problem.conv_problem.GetConv().lowp_quant;
+    const auto lowp_quant = problem.GetConv().lowp_quant;
     const auto isGfx90aFp16altSupport =
-        (ctx.GetStream().GetDeviceName() == "gfx90a") && conv_problem.IsFp16();
+        (ctx.GetStream().GetDeviceName() == "gfx90a") && problem.IsFp16();
 
     const bool need_cast = [&]() {
-        if(problem.conv_problem.GetOut().GetType() == miopenHalf)
+        if(problem.GetOut().GetType() == miopenHalf)
             return use_fp32_global_split_on_fp16;
-        if(problem.conv_problem.GetOut().GetType() == miopenBFloat16)
-            return need_set_zero;
+        if(problem.GetOut().GetType() == miopenBFloat16)
+            return config.gemm_k_global_split > 0;
         return false;
     }();
     const auto is_nchw = problem.IsLayoutDefault();
@@ -569,8 +558,6 @@ InvokerFactory MakeImplGemmDynamicForwardXdlopsNHWCInvokerFactory(
     int trans_weight_idx = -1;
     int trans_output_idx = -1;
 
-    constexpr size_t buf_alignment = 256;
-
     if(is_nchw)
     {
         TransposeSolutionDefault2Nhwc trans_input(ctx, problem.GetInDataType(), n, c, hi, wi);
@@ -609,7 +596,7 @@ InvokerFactory MakeImplGemmDynamicForwardXdlopsNHWCInvokerFactory(
     const size_t cast_size = need_cast ? miopen::GetTypeSize(miopenFloat) * n * k * ho * wo : 0;
 
     MultiBufferWorkspaceTraits wt(
-        {trans_input_size, trans_weight_size, trans_output_size, cast_size}, buf_alignment);
+        {trans_input_size, trans_weight_size, trans_output_size, cast_size});
 
     trans_input_offset  = wt.GetOffset(0);
     trans_weight_offset = wt.GetOffset(1);
@@ -619,9 +606,8 @@ InvokerFactory MakeImplGemmDynamicForwardXdlopsNHWCInvokerFactory(
 
     const int kID_trans_start = isGfx90aFp16altSupport ? 2 : 1;
 
-    const TensorDescriptor cast_desc(miopenFloat,
-                                     problem.conv_problem.GetOut().GetLengths(),
-                                     problem.conv_problem.GetOut().GetStrides());
+    const TensorDescriptor cast_desc(
+        miopenFloat, problem.GetOut().GetLengths(), problem.GetOut().GetStrides());
     auto null_buf = shared<Data_t>{};
 
     return [=](const std::vector<Kernel>& kernels) mutable {
@@ -705,6 +691,7 @@ InvokerFactory MakeImplGemmDynamicForwardXdlopsNHWCInvokerFactory(
             {
                 CastTensor(handle,
                            &lowp_quant,
+                           problem.IsDirectionForward(),
                            cast_desc,
                            cast_buf.get(),
                            tensors.outDesc,
@@ -736,27 +723,26 @@ InvokerFactory MakeImplGemmDynamicForwardXdlopsNHWCInvokerFactory(
 }
 
 InvokerFactory MakeImplGemmDynamicBackwardDataXdlopsNHWCInvokerFactory(
-    const ConvolutionContext& ctx,
-    const miopen::ProblemDescription& problem,
-    const solver::PerformanceConfigAsmImplicitGemmGTCBwdXdlopsNHWC& config)
+    const ExecutionContext& ctx,
+    const ProblemDescription& problem,
+    const solver::conv::PerformanceConfigAsmImplicitGemmGTCBwdXdlopsNHWC& config)
 {
-    const auto& conv_problem = problem.conv_problem;
-    int hi                   = conv_problem.GetOutHeight();
-    int wi                   = conv_problem.GetOutWidth();
-    int n                    = conv_problem.GetInBatchSize();
-    int k                    = conv_problem.GetInChannels();
-    int c                    = conv_problem.GetOutChannels();
-    int ho                   = conv_problem.GetInHeight();
-    int wo                   = conv_problem.GetInWidth();
-    int stride_h   = conv_problem.GetOutHeight() > 1 ? conv_problem.GetKernelStrideH() : 1;
-    int stride_w   = conv_problem.GetOutWidth() > 1 ? conv_problem.GetKernelStrideW() : 1;
-    int dilation_h = conv_problem.GetWeightsHeight() > 1 ? conv_problem.GetDilationH() : 1;
-    int dilation_w = conv_problem.GetWeightsWidth() > 1 ? conv_problem.GetDilationW() : 1;
-    int pad_h      = conv_problem.GetPadH();
-    int pad_w      = conv_problem.GetPadW();
-    int y          = conv_problem.GetWeightsHeight();
-    int x          = conv_problem.GetWeightsWidth();
-    int group      = conv_problem.GetGroupCount();
+    int hi         = problem.GetOutHeight();
+    int wi         = problem.GetOutWidth();
+    int n          = problem.GetInBatchSize();
+    int k          = problem.GetInChannels();
+    int c          = problem.GetOutChannels();
+    int ho         = problem.GetInHeight();
+    int wo         = problem.GetInWidth();
+    int stride_h   = problem.GetOutHeight() > 1 ? problem.GetKernelStrideH() : 1;
+    int stride_w   = problem.GetOutWidth() > 1 ? problem.GetKernelStrideW() : 1;
+    int dilation_h = problem.GetWeightsHeight() > 1 ? problem.GetDilationH() : 1;
+    int dilation_w = problem.GetWeightsWidth() > 1 ? problem.GetDilationW() : 1;
+    int pad_h      = problem.GetPadH();
+    int pad_w      = problem.GetPadW();
+    int y          = problem.GetWeightsHeight();
+    int x          = problem.GetWeightsWidth();
+    int group      = problem.GetGroupCount();
 
     int gcd_stride_dilation_h = solver::gcd(stride_h, dilation_h);
     int gcd_stride_dilation_w = solver::gcd(stride_w, dilation_w);
@@ -807,11 +793,8 @@ InvokerFactory MakeImplGemmDynamicBackwardDataXdlopsNHWCInvokerFactory(
     int dtile_h  = num_of_gemms > 1 ? static_cast<int>(mdiv_group_mn.magic) : h_tilda;
     int dtile_w  = num_of_gemms > 1 ? static_cast<int>(mdiv_group_mn.shift) : w_tilda;
 
-    bool need_set_zero                 = false;
+    bool need_set_zero                 = true;
     bool use_fp32_global_split_on_fp16 = config.vector_store == 1 && config.gemm_k_global_split > 0;
-    if(y < stride_h || x < stride_w || dilation_h != 1 || dilation_w != 1)
-        need_set_zero = true;
-    need_set_zero |= config.gemm_k_global_split > 0;
 
     std::vector<OpKernelArg> opArgs;
     opArgs.emplace_back(0); // placeholder
@@ -859,14 +842,17 @@ InvokerFactory MakeImplGemmDynamicBackwardDataXdlopsNHWCInvokerFactory(
 
     std::vector<std::vector<OpKernelArg>> opArgsTrans;
 
-    const auto lowp_quant = problem.conv_problem.GetConv().lowp_quant;
+    const auto lowp_quant = problem.GetConv().lowp_quant;
     const auto isGfx90aFp16altSupport =
-        (ctx.GetStream().GetDeviceName() == "gfx90a") && conv_problem.IsFp16();
+        (ctx.GetStream().GetDeviceName() == "gfx90a") && problem.IsFp16();
     const bool need_cast = [&]() {
-        if(problem.conv_problem.GetOut().GetType() == miopenHalf)
+        if(problem.GetOut().GetType() == miopenHalf)
             return use_fp32_global_split_on_fp16;
-        if(problem.conv_problem.GetOut().GetType() == miopenBFloat16)
-            return need_set_zero;
+        if(problem.GetOut().GetType() == miopenBFloat16)
+        {
+            return (y < stride_h || x < stride_w || dilation_h != 1 || dilation_w != 1 ||
+                    config.gemm_k_global_split > 0);
+        }
         return false;
     }();
     const auto is_nchw = problem.IsLayoutDefault();
@@ -888,8 +874,6 @@ InvokerFactory MakeImplGemmDynamicBackwardDataXdlopsNHWCInvokerFactory(
     int trans_weight_idx = -1;
     int trans_output_idx = -1;
 
-    constexpr size_t buf_alignment = 256;
-
     if(is_nchw)
     {
         TransposeSolutionNhwc2Default trans_input(ctx, problem.GetOutDataType(), n, c, hi, wi);
@@ -928,7 +912,7 @@ InvokerFactory MakeImplGemmDynamicBackwardDataXdlopsNHWCInvokerFactory(
     const size_t cast_size = need_cast ? miopen::GetTypeSize(miopenFloat) * n * c * hi * wi : 0;
 
     MultiBufferWorkspaceTraits wt(
-        {trans_input_size, trans_weight_size, trans_output_size, cast_size}, buf_alignment);
+        {trans_input_size, trans_weight_size, trans_output_size, cast_size});
 
     trans_input_offset  = wt.GetOffset(0);
     trans_weight_offset = wt.GetOffset(1);
@@ -938,9 +922,8 @@ InvokerFactory MakeImplGemmDynamicBackwardDataXdlopsNHWCInvokerFactory(
 
     const int kID_trans_start = isGfx90aFp16altSupport ? 2 : 1;
 
-    const TensorDescriptor cast_desc(miopenFloat,
-                                     problem.conv_problem.GetOut().GetLengths(),
-                                     problem.conv_problem.GetOut().GetStrides());
+    const TensorDescriptor cast_desc(
+        miopenFloat, problem.GetOut().GetLengths(), problem.GetOut().GetStrides());
     auto null_buf = shared<Data_t>{};
 
     return [=](const std::vector<Kernel>& kernels) mutable {
@@ -1024,6 +1007,7 @@ InvokerFactory MakeImplGemmDynamicBackwardDataXdlopsNHWCInvokerFactory(
             {
                 CastTensor(handle,
                            &lowp_quant,
+                           false,
                            cast_desc,
                            cast_buf.get(),
                            tensors.outDesc,
@@ -1054,27 +1038,26 @@ InvokerFactory MakeImplGemmDynamicBackwardDataXdlopsNHWCInvokerFactory(
 }
 
 InvokerFactory MakeImplGemmDynamicForwardDlopsNCHWCInvokerFactory(
-    const miopen::ProblemDescription& problem,
-    const solver::PerformanceConfigAsmImplicitGemmGTCFwdDlopsNCHWC& config)
+    const ProblemDescription& problem,
+    const solver::conv::PerformanceConfigAsmImplicitGemmGTCFwdDlopsNCHWC& config)
 {
-    const auto& conv_problem = problem.conv_problem;
-    int hi                   = conv_problem.GetInHeight();
-    int wi                   = conv_problem.GetInWidth();
-    int n                    = conv_problem.GetInBatchSize();
-    int k                    = conv_problem.GetOutChannels() * config.vector_c;
-    int c                    = conv_problem.GetInChannels();
-    int ks                   = 1;
-    int ho                   = conv_problem.GetOutHeight();
-    int wo                   = conv_problem.GetOutWidth();
-    int stride_h             = conv_problem.GetKernelStrideH();
-    int stride_w             = conv_problem.GetKernelStrideW();
-    int dilation_h           = conv_problem.GetDilationH();
-    int dilation_w           = conv_problem.GetDilationW();
-    int pad_h                = conv_problem.GetPadH();
-    int pad_w                = conv_problem.GetPadW();
-    int y                    = conv_problem.GetWeightsHeight();
-    int x                    = conv_problem.GetWeightsWidth();
-    int group                = conv_problem.GetGroupCount();
+    int hi         = problem.GetInHeight();
+    int wi         = problem.GetInWidth();
+    int n          = problem.GetInBatchSize();
+    int k          = problem.GetOutChannels() * config.vector_c;
+    int c          = problem.GetInChannels();
+    int ks         = 1;
+    int ho         = problem.GetOutHeight();
+    int wo         = problem.GetOutWidth();
+    int stride_h   = problem.GetKernelStrideH();
+    int stride_w   = problem.GetKernelStrideW();
+    int dilation_h = problem.GetDilationH();
+    int dilation_w = problem.GetDilationW();
+    int pad_h      = problem.GetPadH();
+    int pad_w      = problem.GetPadW();
+    int y          = problem.GetWeightsHeight();
+    int x          = problem.GetWeightsWidth();
+    int group      = problem.GetGroupCount();
 
     // Currentlly we do not tile in H/W dimension, using tile H/W as Ho/Wo, Thus Number of Tile
     // equal to one
