@@ -35,6 +35,7 @@
 #include <miopen/conv/problem_description.hpp>
 #include <miopen/tensor_ops.hpp>
 #include <miopen/driver_arguments.hpp>
+#include <miopen/config.hpp>
 
 #include <algorithm>
 #include <optional>
@@ -113,12 +114,17 @@ static inline auto MakeWrWCtxAndProblem(miopenHandle_t handle,
     return std::make_tuple(std::move(ctx), std::move(problem));
 }
 
+MIOPEN_EXPORT
 extern "C" miopenStatus_t miopenCreateConvolutionDescriptor(miopenConvolutionDescriptor_t* convDesc)
 {
     MIOPEN_LOG_FUNCTION(convDesc);
-    return miopen::try_([&] { miopen::deref(convDesc) = new miopen::ConvolutionDescriptor(); });
+    return miopen::try_([&] {
+        auto& desc = miopen::deref(convDesc);
+        desc       = new miopen::ConvolutionDescriptor();
+    });
 }
 
+MIOPEN_EXPORT
 extern "C" miopenStatus_t miopenInitConvolutionDescriptor(miopenConvolutionDescriptor_t convDesc,
                                                           miopenConvolutionMode_t c_mode,
                                                           int pad_h,
@@ -139,6 +145,7 @@ extern "C" miopenStatus_t miopenInitConvolutionDescriptor(miopenConvolutionDescr
     });
 }
 
+MIOPEN_EXPORT
 extern "C" miopenStatus_t miopenInitConvolutionNdDescriptor(miopenConvolutionDescriptor_t convDesc,
                                                             int spatialDim,
                                                             const int* padA,
@@ -163,6 +170,7 @@ extern "C" miopenStatus_t miopenInitConvolutionNdDescriptor(miopenConvolutionDes
     });
 }
 
+MIOPEN_EXPORT
 extern "C" miopenStatus_t miopenGetConvolutionGroupCount(miopenConvolutionDescriptor_t convDesc,
                                                          int* groupCount)
 {
@@ -170,6 +178,7 @@ extern "C" miopenStatus_t miopenGetConvolutionGroupCount(miopenConvolutionDescri
     return miopen::try_([&] { miopen::deref(groupCount) = miopen::deref(convDesc).group_count; });
 }
 
+MIOPEN_EXPORT
 extern "C" miopenStatus_t miopenSetConvolutionGroupCount(miopenConvolutionDescriptor_t convDesc,
                                                          int groupCount)
 {
@@ -177,6 +186,7 @@ extern "C" miopenStatus_t miopenSetConvolutionGroupCount(miopenConvolutionDescri
     return miopen::try_([&] { miopen::deref(convDesc).group_count = groupCount; });
 }
 
+MIOPEN_EXPORT
 extern "C" miopenStatus_t miopenSetConvolutionFindMode(miopenConvolutionDescriptor_t convDesc,
                                                        miopenConvolutionFindMode_t findMode)
 {
@@ -186,6 +196,7 @@ extern "C" miopenStatus_t miopenSetConvolutionFindMode(miopenConvolutionDescript
     });
 }
 
+MIOPEN_EXPORT
 extern "C" miopenStatus_t miopenGetConvolutionFindMode(const miopenConvolutionDescriptor_t convDesc,
                                                        miopenConvolutionFindMode_t* findMode)
 {
@@ -196,14 +207,77 @@ extern "C" miopenStatus_t miopenGetConvolutionFindMode(const miopenConvolutionDe
     });
 }
 
+MIOPEN_EXPORT extern "C" miopenStatus_t
+miopenConvolutionABBackwardWeightsGetWorkSpaceSize(const miopenAlphaBetaCase_t alpha_beta_case,
+                                                   const miopenTensorDescriptor_t inputTensorDesc,
+                                                   const miopenTensorDescriptor_t outputTensorDesc,
+                                                   const miopenConvolutionDescriptor_t convDesc,
+                                                   size_t* buffer_size)
+{
+    MIOPEN_LOG_FUNCTION(alpha_beta_case, outputTensorDesc);
+    return miopen::try_([&] {
+        miopenDataType_t data_type = miopen::deref(outputTensorDesc).GetType();
+        size_t in_spatial_dims     = miopen::deref(inputTensorDesc).GetNumDims();
+
+        assert(in_spatial_dims == miopen::deref(outputTensorDesc).GetNumDims());
+
+        int G    = miopen::deref(convDesc).GetGroupCount();
+        size_t C = std::get<1>(
+            miopen::GetNCDHW(in_spatial_dims, miopen::deref(inputTensorDesc).GetLengths()));
+        size_t K = std::get<1>(
+            miopen::GetNCDHW(in_spatial_dims, miopen::deref(outputTensorDesc).GetLengths()));
+
+        auto CKWrwRequireWorkspace = [&](size_t G,
+                                         size_t C,
+                                         size_t K,
+                                         miopenDataType_t data_type,
+                                         miopenAlphaBetaCase_t alpha_beta_case) {
+            auto is_odd        = [](int num) { return num % 2 != 0; };
+            size_t C_per_group = C / G;
+            size_t K_per_group = K / G;
+
+            return (alpha_beta_case == BILINEAR || alpha_beta_case == SCALE) ||
+                   (data_type == miopenHalf && (is_odd(C_per_group) || is_odd(K_per_group)));
+        };
+
+        size_t output_tensor_size = miopen::deref(outputTensorDesc).GetElementSize();
+        size_t byte_size          = 0;
+        if(CKWrwRequireWorkspace(G, C, K, data_type, alpha_beta_case))
+        {
+            switch(data_type)
+            {
+            case miopenInt32:
+            case miopenFloat:
+            case miopenHalf:
+            case miopenBFloat16:
+            case miopenInt8:
+            case miopenFloat8:
+            case miopenBFloat8: byte_size = 4; break;
+            case miopenDouble:
+            case miopenInt64: byte_size = 8; break;
+            }
+            *buffer_size = byte_size * output_tensor_size;
+        }
+        else
+        {
+            *buffer_size = 0;
+        }
+
+        MIOPEN_LOG_FUNCTION(
+            alpha_beta_case, data_type, C, K, output_tensor_size, byte_size, *buffer_size);
+    });
+}
+
 // Hidden C++ functions for MIGraphX.
-extern "C" MIOPEN_EXPORT miopenStatus_t
+MIOPEN_EXPORT extern "C" miopenStatus_t
 miopenHiddenSetConvolutionFindMode(miopenConvolutionDescriptor_t convDesc, int findMode)
 {
     return miopen::try_([&] {
         miopen::deref(convDesc).findMode.Set(static_cast<miopen::FindMode::Values>(findMode));
     });
 }
+
+MIOPEN_EXPORT
 extern "C" miopenStatus_t miopenHiddenGetConvolutionFindMode(miopenConvolutionDescriptor_t convDesc,
                                                              int* findMode)
 {
@@ -212,7 +286,7 @@ extern "C" miopenStatus_t miopenHiddenGetConvolutionFindMode(miopenConvolutionDe
     });
 }
 
-extern "C" miopenStatus_t
+MIOPEN_EXPORT extern "C" miopenStatus_t
 miopenSetTransposeConvOutputPadding(miopenConvolutionDescriptor_t convDesc, int adj_h, int adj_w)
 {
     MIOPEN_LOG_FUNCTION(convDesc, adj_h, adj_w);
@@ -227,6 +301,7 @@ miopenSetTransposeConvOutputPadding(miopenConvolutionDescriptor_t convDesc, int
     });
 }
 
+MIOPEN_EXPORT
 extern "C" miopenStatus_t miopenSetTransposeConvNdOutputPadding(
     miopenConvolutionDescriptor_t convDesc, int spatialDim, const int* adjA)
 {
@@ -245,6 +320,7 @@ extern "C" miopenStatus_t miopenSetTransposeConvNdOutputPadding(
     });
 }
 
+MIOPEN_EXPORT
 extern "C" miopenStatus_t miopenGetConvolutionDescriptor(miopenConvolutionDescriptor_t convDesc,
                                                          miopenConvolutionMode_t* c_mode,
                                                          int* pad_h,
@@ -271,6 +347,7 @@ extern "C" miopenStatus_t miopenGetConvolutionDescriptor(miopenConvolutionDescri
     });
 }
 
+MIOPEN_EXPORT
 extern "C" miopenStatus_t miopenGetConvolutionNdDescriptor(miopenConvolutionDescriptor_t convDesc,
                                                            int requestedSpatialDim,
                                                            int* spatialDim,
@@ -301,6 +378,7 @@ extern "C" miopenStatus_t miopenGetConvolutionNdDescriptor(miopenConvolutionDesc
     });
 }
 
+MIOPEN_EXPORT
 extern "C" miopenStatus_t miopenGetConvolutionSpatialDim(miopenConvolutionDescriptor_t convDesc,
                                                          int* spatialDim)
 {
@@ -309,7 +387,7 @@ extern "C" miopenStatus_t miopenGetConvolutionSpatialDim(miopenConvolutionDescri
         [&] { miopen::deref(spatialDim) = miopen::deref(convDesc).GetSpatialDimension(); });
 }
 
-extern "C" miopenStatus_t
+MIOPEN_EXPORT extern "C" miopenStatus_t
 miopenGetConvolutionForwardOutputDim(miopenConvolutionDescriptor_t convDesc,
                                      const miopenTensorDescriptor_t inputTensorDesc,
                                      const miopenTensorDescriptor_t filterDesc,
@@ -332,7 +410,7 @@ miopenGetConvolutionForwardOutputDim(miopenConvolutionDescriptor_t convDesc,
     });
 }
 
-extern "C" miopenStatus_t
+MIOPEN_EXPORT extern "C" miopenStatus_t
 miopenGetConvolutionNdForwardOutputDim(miopenConvolutionDescriptor_t convDesc,
                                        const miopenTensorDescriptor_t inputTensorDesc,
                                        const miopenTensorDescriptor_t filterDesc,
@@ -344,22 +422,23 @@ miopenGetConvolutionNdForwardOutputDim(miopenConvolutionDescriptor_t convDesc,
         auto out_desc = miopen::deref(convDesc).GetForwardOutputTensor(
             miopen::deref(inputTensorDesc), miopen::deref(filterDesc));
 
-        miopen::deref(nDim) = out_desc.GetSize();
+        miopen::deref(nDim) = out_desc.GetNumDims();
 
-        for(int i = 0; i < out_desc.GetSize(); ++i)
+        for(unsigned i = 0; i < out_desc.GetNumDims(); ++i)
         {
             outputTensorDimA[i] = out_desc.GetLengths()[i];
         }
     });
 }
 
-extern "C" miopenStatus_t miopenDestroyConvolutionDescriptor(miopenConvolutionDescriptor_t convDesc)
+MIOPEN_EXPORT extern "C" miopenStatus_t
+miopenDestroyConvolutionDescriptor(miopenConvolutionDescriptor_t convDesc)
 {
     MIOPEN_LOG_FUNCTION(convDesc);
     return miopen::try_([&] { miopen_destroy_object(convDesc); });
 }
 
-extern "C" miopenStatus_t
+MIOPEN_EXPORT extern "C" miopenStatus_t
 miopenConvolutionForwardGetWorkSpaceSize(miopenHandle_t handle,
                                          const miopenTensorDescriptor_t wDesc,
                                          const miopenTensorDescriptor_t xDesc,
@@ -380,7 +459,7 @@ miopenConvolutionForwardGetWorkSpaceSize(miopenHandle_t handle,
 namespace miopen {
 namespace debug {
 
-MIOPEN_EXPORT
+MIOPEN_INTERNALS_EXPORT
 void LogCmdConvolution(const miopen::TensorDescriptor& x,
                        const miopen::TensorDescriptor& w,
                        const miopen::ConvolutionDescriptor& conv,
@@ -395,7 +474,7 @@ void LogCmdConvolution(const miopen::TensorDescriptor& x,
     }
 }
 
-MIOPEN_EXPORT
+MIOPEN_INTERNALS_EXPORT
 void LogCmdFindConvolution(const miopen::TensorDescriptor& x,
                            const miopen::TensorDescriptor& w,
                            const miopen::ConvolutionDescriptor& conv,
@@ -410,7 +489,7 @@ void LogCmdFindConvolution(const miopen::TensorDescriptor& x,
     }
 }
 
-MIOPEN_EXPORT
+MIOPEN_INTERNALS_EXPORT
 void LogCmdConvolution(const miopenTensorDescriptor_t& xDesc,
                        const miopenTensorDescriptor_t& wDesc,
                        const miopenConvolutionDescriptor_t& convDesc,
@@ -427,7 +506,7 @@ void LogCmdConvolution(const miopenTensorDescriptor_t& xDesc,
     }
 }
 
-MIOPEN_EXPORT
+MIOPEN_INTERNALS_EXPORT
 void LogCmdFindConvolution(const miopenTensorDescriptor_t& xDesc,
                            const miopenTensorDescriptor_t& wDesc,
                            const miopenConvolutionDescriptor_t& convDesc,
@@ -447,7 +526,7 @@ void LogCmdFindConvolution(const miopenTensorDescriptor_t& xDesc,
 } // namespace debug
 } // namespace miopen
 
-extern "C" miopenStatus_t
+MIOPEN_EXPORT extern "C" miopenStatus_t
 miopenFindConvolutionForwardAlgorithm(miopenHandle_t handle,
                                       const miopenTensorDescriptor_t xDesc,
                                       const void* x,
@@ -525,19 +604,20 @@ miopenFindConvolutionForwardAlgorithm(miopenHandle_t handle,
     });
 }
 
-extern "C" miopenStatus_t miopenConvolutionForward(miopenHandle_t handle,
-                                                   const void* alpha,
-                                                   const miopenTensorDescriptor_t xDesc,
-                                                   const void* x,
-                                                   const miopenTensorDescriptor_t wDesc,
-                                                   const void* w,
-                                                   const miopenConvolutionDescriptor_t convDesc,
-                                                   miopenConvFwdAlgorithm_t algo,
-                                                   const void* beta,
-                                                   const miopenTensorDescriptor_t yDesc,
-                                                   void* y,
-                                                   void* workSpace,
-                                                   size_t workSpaceSize)
+MIOPEN_EXPORT extern "C" miopenStatus_t
+miopenConvolutionForward(miopenHandle_t handle,
+                         const void* alpha,
+                         const miopenTensorDescriptor_t xDesc,
+                         const void* x,
+                         const miopenTensorDescriptor_t wDesc,
+                         const void* w,
+                         const miopenConvolutionDescriptor_t convDesc,
+                         miopenConvFwdAlgorithm_t algo,
+                         const void* beta,
+                         const miopenTensorDescriptor_t yDesc,
+                         void* y,
+                         void* workSpace,
+                         size_t workSpaceSize)
 {
 
     MIOPEN_LOG_FUNCTION(handle,
@@ -592,13 +672,14 @@ extern "C" miopenStatus_t miopenConvolutionForward(miopenHandle_t handle,
     });
 }
 
-extern "C" miopenStatus_t miopenConvolutionForwardBias(miopenHandle_t handle,
-                                                       const void* alpha,
-                                                       const miopenTensorDescriptor_t bDesc,
-                                                       const void* b,
-                                                       const void* beta,
-                                                       const miopenTensorDescriptor_t yDesc,
-                                                       void* y)
+MIOPEN_EXPORT extern "C" miopenStatus_t
+miopenConvolutionForwardBias(miopenHandle_t handle,
+                             const void* alpha,
+                             const miopenTensorDescriptor_t bDesc,
+                             const void* b,
+                             const void* beta,
+                             const miopenTensorDescriptor_t yDesc,
+                             void* y)
 {
 
     MIOPEN_LOG_FUNCTION(handle, alpha, bDesc, b, beta, yDesc, y);
@@ -625,7 +706,7 @@ extern "C" miopenStatus_t miopenConvolutionForwardBias(miopenHandle_t handle,
     });
 }
 
-extern "C" miopenStatus_t
+MIOPEN_EXPORT extern "C" miopenStatus_t
 miopenConvolutionForwardGetSolutionCount(miopenHandle_t handle,
                                          const miopenTensorDescriptor_t wDesc,
                                          const miopenTensorDescriptor_t xDesc,
@@ -656,7 +737,7 @@ static inline void ReturnSolutions(const std::vector<miopenConvSolution_t>& solu
     }
 }
 
-extern "C" miopenStatus_t
+MIOPEN_EXPORT extern "C" miopenStatus_t
 miopenConvolutionForwardGetSolution(miopenHandle_t handle,
                                     const miopenTensorDescriptor_t wDesc,
                                     const miopenTensorDescriptor_t xDesc,
@@ -680,7 +761,7 @@ miopenConvolutionForwardGetSolution(miopenHandle_t handle,
     });
 }
 
-extern "C" miopenStatus_t
+MIOPEN_EXPORT extern "C" miopenStatus_t
 miopenConvolutionForwardGetSolutionWorkspaceSize(miopenHandle_t handle,
                                                  const miopenTensorDescriptor_t wDesc,
                                                  const miopenTensorDescriptor_t xDesc,
@@ -712,7 +793,7 @@ miopenConvolutionForwardGetSolutionWorkspaceSize(miopenHandle_t handle,
     });
 }
 
-extern "C" miopenStatus_t
+MIOPEN_EXPORT extern "C" miopenStatus_t
 miopenConvolutionForwardCompileSolution(miopenHandle_t handle,
                                         const miopenTensorDescriptor_t wDesc,
                                         const miopenTensorDescriptor_t xDesc,
@@ -729,7 +810,7 @@ miopenConvolutionForwardCompileSolution(miopenHandle_t handle,
     });
 }
 
-extern "C" miopenStatus_t
+MIOPEN_EXPORT extern "C" miopenStatus_t
 miopenConvolutionForwardImmediate(miopenHandle_t handle,
                                   const miopenTensorDescriptor_t wDesc,
                                   const void* w,
@@ -777,7 +858,7 @@ miopenConvolutionForwardImmediate(miopenHandle_t handle,
     });
 }
 
-extern "C" miopenStatus_t
+MIOPEN_EXPORT extern "C" miopenStatus_t
 miopenConvolutionBackwardDataGetSolutionCount(miopenHandle_t handle,
                                               const miopenTensorDescriptor_t dyDesc,
                                               const miopenTensorDescriptor_t wDesc,
@@ -795,7 +876,7 @@ miopenConvolutionBackwardDataGetSolutionCount(miopenHandle_t handle,
     });
 }
 
-extern "C" miopenStatus_t
+MIOPEN_EXPORT extern "C" miopenStatus_t
 miopenConvolutionBackwardDataGetSolution(miopenHandle_t handle,
                                          const miopenTensorDescriptor_t dyDesc,
                                          const miopenTensorDescriptor_t wDesc,
@@ -819,7 +900,7 @@ miopenConvolutionBackwardDataGetSolution(miopenHandle_t handle,
     });
 }
 
-extern "C" miopenStatus_t
+MIOPEN_EXPORT extern "C" miopenStatus_t
 miopenConvolutionBackwardDataGetSolutionWorkspaceSize(miopenHandle_t handle,
                                                       const miopenTensorDescriptor_t dyDesc,
                                                       const miopenTensorDescriptor_t wDesc,
@@ -851,7 +932,7 @@ miopenConvolutionBackwardDataGetSolutionWorkspaceSize(miopenHandle_t handle,
     });
 }
 
-extern "C" miopenStatus_t
+MIOPEN_EXPORT extern "C" miopenStatus_t
 miopenConvolutionBackwardDataCompileSolution(miopenHandle_t handle,
                                              const miopenTensorDescriptor_t dyDesc,
                                              const miopenTensorDescriptor_t wDesc,
@@ -868,7 +949,7 @@ miopenConvolutionBackwardDataCompileSolution(miopenHandle_t handle,
     });
 }
 
-extern "C" miopenStatus_t
+MIOPEN_EXPORT extern "C" miopenStatus_t
 miopenConvolutionBackwardDataImmediate(miopenHandle_t handle,
                                        const miopenTensorDescriptor_t dyDesc,
                                        const void* dy,
@@ -915,7 +996,7 @@ miopenConvolutionBackwardDataImmediate(miopenHandle_t handle,
     });
 }
 
-extern "C" miopenStatus_t
+MIOPEN_EXPORT extern "C" miopenStatus_t
 miopenConvolutionBackwardWeightsGetSolutionCount(miopenHandle_t handle,
                                                  const miopenTensorDescriptor_t dyDesc,
                                                  const miopenTensorDescriptor_t xDesc,
@@ -933,7 +1014,7 @@ miopenConvolutionBackwardWeightsGetSolutionCount(miopenHandle_t handle,
     });
 }
 
-extern "C" miopenStatus_t
+MIOPEN_EXPORT extern "C" miopenStatus_t
 miopenConvolutionBackwardWeightsGetSolution(miopenHandle_t handle,
                                             const miopenTensorDescriptor_t dyDesc,
                                             const miopenTensorDescriptor_t xDesc,
@@ -957,7 +1038,7 @@ miopenConvolutionBackwardWeightsGetSolution(miopenHandle_t handle,
     });
 }
 
-extern "C" miopenStatus_t miopenConvolutionBackwardWeightsGetSolutionWorkspaceSize(
+MIOPEN_EXPORT extern "C" miopenStatus_t miopenConvolutionBackwardWeightsGetSolutionWorkspaceSize(
     miopenHandle_t handle,
     const miopenTensorDescriptor_t dyDesc,
     const miopenTensorDescriptor_t xDesc,
@@ -989,7 +1070,7 @@ extern "C" miopenStatus_t miopenConvolutionBackwardWeightsGetSolutionWorkspaceSi
     });
 }
 
-extern "C" miopenStatus_t
+MIOPEN_EXPORT extern "C" miopenStatus_t
 miopenConvolutionBackwardWeightsCompileSolution(miopenHandle_t handle,
                                                 const miopenTensorDescriptor_t dyDesc,
                                                 const miopenTensorDescriptor_t xDesc,
@@ -1006,7 +1087,7 @@ miopenConvolutionBackwardWeightsCompileSolution(miopenHandle_t handle,
     });
 }
 
-extern "C" miopenStatus_t
+MIOPEN_EXPORT extern "C" miopenStatus_t
 miopenConvolutionBackwardWeightsImmediate(miopenHandle_t handle,
                                           const miopenTensorDescriptor_t dyDesc,
                                           const void* dy,
@@ -1053,7 +1134,7 @@ miopenConvolutionBackwardWeightsImmediate(miopenHandle_t handle,
     });
 }
 
-extern "C" miopenStatus_t
+MIOPEN_EXPORT extern "C" miopenStatus_t
 miopenFindConvolutionBackwardDataAlgorithm(miopenHandle_t handle,
                                            const miopenTensorDescriptor_t dyDesc,
                                            const void* dy,
@@ -1131,7 +1212,7 @@ miopenFindConvolutionBackwardDataAlgorithm(miopenHandle_t handle,
     });
 }
 
-extern "C" miopenStatus_t
+MIOPEN_EXPORT extern "C" miopenStatus_t
 miopenConvolutionBackwardData(miopenHandle_t handle,
                               const void* alpha,
                               const miopenTensorDescriptor_t dyDesc,
@@ -1199,7 +1280,7 @@ miopenConvolutionBackwardData(miopenHandle_t handle,
     });
 }
 
-extern "C" miopenStatus_t
+MIOPEN_EXPORT extern "C" miopenStatus_t
 miopenConvolutionBackwardDataGetWorkSpaceSize(miopenHandle_t handle,
                                               const miopenTensorDescriptor_t dyDesc,
                                               const miopenTensorDescriptor_t wDesc,
@@ -1216,7 +1297,7 @@ miopenConvolutionBackwardDataGetWorkSpaceSize(miopenHandle_t handle,
     });
 }
 
-extern "C" miopenStatus_t
+MIOPEN_EXPORT extern "C" miopenStatus_t
 miopenConvolutionBackwardWeightsGetWorkSpaceSize(miopenHandle_t handle,
                                                  const miopenTensorDescriptor_t dyDesc,
                                                  const miopenTensorDescriptor_t xDesc,
@@ -1233,7 +1314,7 @@ miopenConvolutionBackwardWeightsGetWorkSpaceSize(miopenHandle_t handle,
     });
 }
 
-extern "C" miopenStatus_t
+MIOPEN_EXPORT extern "C" miopenStatus_t
 miopenFindConvolutionBackwardWeightsAlgorithm(miopenHandle_t handle,
                                               const miopenTensorDescriptor_t dyDesc,
                                               const void* dy,
@@ -1286,7 +1367,7 @@ miopenFindConvolutionBackwardWeightsAlgorithm(miopenHandle_t handle,
     });
 }
 
-extern "C" miopenStatus_t
+MIOPEN_EXPORT extern "C" miopenStatus_t
 miopenConvolutionBackwardWeights(miopenHandle_t handle,
                                  const void* alpha,
                                  const miopenTensorDescriptor_t dyDesc,
@@ -1336,6 +1417,7 @@ miopenConvolutionBackwardWeights(miopenHandle_t handle,
     });
 }
 
+MIOPEN_EXPORT
 extern "C" miopenStatus_t miopenConvolutionBackwardBias(miopenHandle_t handle,
                                                         const void* alpha,
                                                         const miopenTensorDescriptor_t dyDesc,
@@ -1363,6 +1445,7 @@ extern "C" miopenStatus_t miopenConvolutionBackwardBias(miopenHandle_t handle,
     });
 }
 
+MIOPEN_EXPORT
 extern "C" miopenStatus_t miopenSetConvolutionAttribute(miopenConvolutionDescriptor_t convDesc,
                                                         const miopenConvolutionAttrib_t attr,
                                                         const int value)
@@ -1371,6 +1454,7 @@ extern "C" miopenStatus_t miopenSetConvolutionAttribute(miopenConvolutionDescrip
     return miopen::try_([&] { miopen::deref(convDesc).attribute.Set(attr, value); });
 }
 
+MIOPEN_EXPORT
 extern "C" miopenStatus_t miopenGetConvolutionAttribute(miopenConvolutionDescriptor_t convDesc,
                                                         const miopenConvolutionAttrib_t attr,
                                                         int* const value)
