@@ -67,7 +67,7 @@ struct ExecutionContext;
 struct Handle;
 struct TensorDescriptor;
 
-struct ConvolutionAttribute
+struct MIOPEN_INTERNALS_EXPORT ConvolutionAttribute
 {
     class Gfx90aFp16alt
     {
@@ -76,9 +76,7 @@ struct ConvolutionAttribute
 
         inline int Get() const
         {
-            if(!miopen::IsUnset(ENV(MIOPEN_DEBUG_CONVOLUTION_ATTRIB_FP16_ALT_IMPL)))
-                return miopen::Value(ENV(MIOPEN_DEBUG_CONVOLUTION_ATTRIB_FP16_ALT_IMPL));
-            return value;
+            return env::value_or(MIOPEN_DEBUG_CONVOLUTION_ATTRIB_FP16_ALT_IMPL, value);
         }
 
     public:
@@ -105,18 +103,13 @@ struct ConvolutionAttribute
 
         inline miopenF8RoundingMode_t Get() const
         {
-            if(!miopen::IsUnset(ENV(MIOPEN_DEBUG_CONVOLUTION_ATTRIB_FP8_ROUNDING_MODE)))
-                return static_cast<miopenF8RoundingMode_t>(
-                    miopen::Value(ENV(MIOPEN_DEBUG_CONVOLUTION_ATTRIB_FP8_ROUNDING_MODE)));
-            return rounding_mode;
+            return env::value_or(MIOPEN_DEBUG_CONVOLUTION_ATTRIB_FP8_ROUNDING_MODE, rounding_mode);
         }
 
         inline uint32_t GetSeed() const
         {
             // assert(rounding_mode == miopenF8RoundingModeStochastic);
-            if(!miopen::IsUnset(ENV(MIOPEN_DEBUG_CONVOLUTION_ATTRIB_FP8_ROUNDING_SEED)))
-                return miopen::Value(ENV(MIOPEN_DEBUG_CONVOLUTION_ATTRIB_FP8_ROUNDING_SEED));
-            return seed;
+            return env::value_or(MIOPEN_DEBUG_CONVOLUTION_ATTRIB_FP8_ROUNDING_SEED, seed);
         }
 
         inline void SetSeed(const uint32_t s) { seed = s; }
@@ -130,10 +123,7 @@ struct ConvolutionAttribute
     public:
         inline int Get() const
         {
-            if(!miopen::IsUnset(ENV(MIOPEN_DEBUG_CONVOLUTION_DETERMINISTIC)))
-                return static_cast<int>(
-                    miopen::IsEnabled(ENV(MIOPEN_DEBUG_CONVOLUTION_DETERMINISTIC)));
-            return value;
+            return env::value_or(MIOPEN_DEBUG_CONVOLUTION_DETERMINISTIC, value);
         }
         operator bool() const
         {
@@ -153,7 +143,15 @@ struct ConvolutionAttribute
     friend void from_json(const nlohmann::json& json, ConvolutionAttribute& conv);
 };
 
-struct MIOPEN_EXPORT ConvolutionDescriptor : miopenConvolutionDescriptor
+struct Solution;
+
+std::vector<Solution> FindConvolution(const ExecutionContext& ctx,
+                                      const conv::ProblemDescription& problem,
+                                      const AnyInvokeParams& invoke_ctx,
+                                      int requestAlgoCount,
+                                      bool force_attach_binary);
+
+struct MIOPEN_INTERNALS_EXPORT ConvolutionDescriptor : miopenConvolutionDescriptor
 {
     ConvolutionDescriptor(std::size_t spatial_dim,
                           miopenConvolutionMode_t c_mode,
@@ -229,10 +227,12 @@ struct MIOPEN_EXPORT ConvolutionDescriptor : miopenConvolutionDescriptor
     std::size_t GetSolutionCount(const ExecutionContext& ctx,
                                  const conv::ProblemDescription& problem) const;
 
-    std::vector<miopenConvSolution_t> GetSolutions(const ExecutionContext& ctx,
-                                                   const conv::ProblemDescription& problem,
-                                                   size_t maxSolutionCount,
-                                                   bool* fallbackPathTaken) const;
+    std::vector<miopenConvSolution_t>
+    GetSolutions(const ExecutionContext& ctx,
+                 const conv::ProblemDescription& problem,
+                 size_t maxSolutionCount,
+                 bool* fallbackPathTaken,
+                 const AnyInvokeParams* invokeParams = nullptr) const;
 
     void CompileSolution(const ExecutionContext& ctx,
                          const conv::ProblemDescription& problem,
@@ -355,9 +355,11 @@ struct MIOPEN_EXPORT ConvolutionDescriptor : miopenConvolutionDescriptor
     FindMode findMode;
     ConvolutionAttribute attribute;
 
-    std::vector<miopenConvSolution_t> GetSolutionsFallback(const ExecutionContext& ctx,
-                                                           const conv::ProblemDescription& problem,
-                                                           size_t maxSolutionCount) const;
+    std::vector<miopenConvSolution_t>
+    GetSolutionsFallback(const ExecutionContext& ctx,
+                         const conv::ProblemDescription& problem,
+                         size_t maxSolutionCount,
+                         const AnyInvokeParams* invokeParams = nullptr) const;
 
     std::size_t GetSolutionCountFallback(const ExecutionContext& ctx,
                                          const conv::ProblemDescription& problem) const;
@@ -369,24 +371,25 @@ private:
     void ValidateTensors(const ConvTensors& conv_tensors) const;
 };
 
-void ConvolutionBackwardBias(const Handle& handle,
-                             const void* alpha,
-                             const TensorDescriptor& dyDesc,
-                             ConstData_t dy,
-                             const void* beta,
-                             const TensorDescriptor& dbDesc,
-                             Data_t db);
+MIOPEN_INTERNALS_EXPORT void ConvolutionBackwardBias(const Handle& handle,
+                                                     const void* alpha,
+                                                     const TensorDescriptor& dyDesc,
+                                                     ConstData_t dy,
+                                                     const void* beta,
+                                                     const TensorDescriptor& dbDesc,
+                                                     Data_t db);
 
-Invoker LoadOrPrepareInvoker(const ExecutionContext& ctx,
-                             const conv::ProblemDescription& problem,
-                             solver::Id solver_id);
+MIOPEN_INTERNALS_EXPORT Invoker LoadOrPrepareInvoker(const ExecutionContext& ctx,
+                                                     const conv::ProblemDescription& problem,
+                                                     solver::Id solver_id);
 
-std::ostream& operator<<(std::ostream& stream, const ConvolutionDescriptor& c);
+MIOPEN_INTERNALS_EXPORT std::ostream& operator<<(std::ostream& stream,
+                                                 const ConvolutionDescriptor& c);
 
-void DumpTensorToFileFromDevice(const miopen::Handle& handle,
-                                const miopen::TensorDescriptor& tDesc,
-                                ConstData_t dData,
-                                const std::string& filename);
+MIOPEN_INTERNALS_EXPORT void DumpTensorToFileFromDevice(const miopen::Handle& handle,
+                                                        const miopen::TensorDescriptor& tDesc,
+                                                        ConstData_t dData,
+                                                        const fs::path& filename);
 
 } // namespace miopen
 
