@@ -27,16 +27,21 @@
 #ifndef MIOPEN_GUARD_MLOPEN_FIND_SOLUTION_HPP
 #define MIOPEN_GUARD_MLOPEN_FIND_SOLUTION_HPP
 
+#include "miopen/miopen.h"
 #include <miopen/env.hpp>
+#include <miopen/errors.hpp>
 #include <miopen/conv_solution.hpp>
 #include <miopen/execution_context.hpp>
 #include <miopen/find_controls.hpp>
 #include <miopen/handle.hpp>
+#include <miopen/mlo_internal.hpp>
 #include <miopen/search_options.hpp>
 #include <miopen/solver_id.hpp>
 #include <miopen/solver.hpp>
 
 #include <limits>
+#include <type_traits>
+#include <optional>
 #include <vector>
 
 namespace miopen {
@@ -50,12 +55,15 @@ auto FindSolutionImpl(rank<1>,
                       Solver s,
                       const Context& context,
                       const Problem& problem,
-                      Db& db,
+                      Db&& db,
                       const AnyInvokeParams& invoke_ctx,
                       const std::string& perf_cfg,
                       const std::optional<FindOptions>& options)
     -> decltype(s.GetSolution(context, problem, s.Search(context, problem, invoke_ctx)))
 {
+    static_assert(std::is_invocable_v<Db>,
+                  "db is meant to be a functor returning a reference to perfdb");
+
     const FindEnforce enforce =
         options && options->find_enforce ? *options->find_enforce : FindEnforce{};
     if(context.disable_perfdb_access)
@@ -66,7 +74,7 @@ auto FindSolutionImpl(rank<1>,
     MIOPEN_LOG_I(s.SolverDbId());
     if(enforce.IsDbClean(context))
     {
-        if(db.Remove(problem, s.SolverDbId()))
+        if(db().Remove(problem, s.SolverDbId()))
             MIOPEN_LOG_W("Perf Db: record removed: " << s.SolverDbId() << ", enforce: " << enforce);
     }
     else
@@ -91,7 +99,7 @@ auto FindSolutionImpl(rank<1>,
                 MIOPEN_LOG_WE("Invalid config loaded from Perf Db: "
                               << s.SolverDbId() << ": " << config << ". Performance may degrade.");
             }
-            else if(db.Load(problem, s.SolverDbId(), config))
+            else if(db().Load(problem, s.SolverDbId(), config))
             {
                 MIOPEN_LOG_I2("Perf Db: record loaded: " << s.SolverDbId());
                 if(s.IsValidPerformanceConfig(context, problem, config))
@@ -101,7 +109,7 @@ auto FindSolutionImpl(rank<1>,
                 MIOPEN_LOG_WE("Invalid config loaded from Perf Db: "
                               << s.SolverDbId() << ": " << config << ". Performance may degrade.");
             }
-            else if(!s.AltSolverDbId().empty() && db.Load(problem, s.AltSolverDbId(), config))
+            else if(!s.AltSolverDbId().empty() && db().Load(problem, s.AltSolverDbId(), config))
             {
                 MIOPEN_LOG_I("Perf Db: alternate record loaded: " << s.AltSolverDbId());
                 if(s.IsValidPerformanceConfig(context, problem, config))
@@ -124,7 +132,7 @@ auto FindSolutionImpl(rank<1>,
             try
             {
                 auto c = s.Search(context, problem, invoke_ctx);
-                db.Update(problem, s.SolverDbId(), c);
+                db().Update(problem, s.SolverDbId(), c);
                 return s.GetSolution(context, problem, c);
             }
             catch(const miopen::Exception& ex)
@@ -143,7 +151,7 @@ auto FindSolutionImpl(rank<0>,
                       Solver s,
                       const Context& context,
                       const Problem& problem,
-                      Db&,
+                      Db&&,
                       const AnyInvokeParams&,
                       const std::string&,
                       const std::optional<FindOptions>&)
@@ -153,6 +161,38 @@ auto FindSolutionImpl(rank<0>,
     return s.GetSolution(context, problem);
 }
 
+template <class Solver, class Context, class Problem>
+auto GetInvokeFactoryImpl(
+    rank<1>, Solver s, const Context& context, const Problem& problem, const std::string& perf_cfg)
+    -> decltype(s.GetInvokerFactory(context,
+                                    problem,
+                                    s.GetDefaultPerformanceConfig(context, problem)))
+{
+    if(!perf_cfg.empty())
+    {
+        using PerformanceConfig = decltype(s.GetDefaultPerformanceConfig(context, problem));
+        PerformanceConfig config{};
+        config.Deserialize(perf_cfg);
+        if(s.IsValidPerformanceConfig(context, problem, config))
+        {
+            return s.GetInvokerFactory(context, problem, config);
+        }
+        MIOPEN_LOG_WE("Invalid config loaded from Perf Db: " << s.SolverDbId() << ": " << config
+                                                             << ". Performance may degrade.");
+    }
+
+    return s.GetInvokerFactory(context, problem, s.GetDefaultPerformanceConfig(context, problem));
+}
+
+template <class Solver, class Context, class Problem>
+auto GetInvokeFactoryImpl(
+    rank<0>, Solver s, const Context& context, const Problem& problem, const std::string&)
+    -> decltype(s.GetInvokerFactory(context, problem))
+{
+    MIOPEN_LOG_I(s.SolverDbId() << " (not searchable)");
+    return s.GetInvokerFactory(context, problem);
+}
+
 /// Finds optimized Solution. Generic method.
 ///
 /// Given the specific problem config, finds (hopefully) optimal
@@ -163,20 +203,40 @@ template <class Solver, class Context, class Problem, class Db>
 ConvSolution FindSolution(Solver s,
                           const Context& context,
                           const Problem& problem,
-                          Db& db,
+                          Db&& db,
                           const AnyInvokeParams& invoke_ctx,
                           const std::string& perf_cfg               = "",
                           const std::optional<FindOptions>& options = std::nullopt)
 {
     static_assert(sizeof(Solver) == sizeof(SolverBase), "Solver must be stateless");
     static_assert(std::is_base_of<SolverBase, Solver>{}, "Not derived class of SolverBase");
+
+    decltype(auto) db_getter = [&]() -> decltype(auto) {
+        if constexpr(std::is_invocable_v<Db>)
+            return db;
+        else
+            return [&]() -> std::decay_t<Db>& { return db; };
+    }();
+
     // TODO: This assumes all solutions are ConvSolution
     auto solution =
-        FindSolutionImpl(rank<1>{}, s, context, problem, db, invoke_ctx, perf_cfg, options);
+        FindSolutionImpl(rank<1>{}, s, context, problem, db_getter, invoke_ctx, perf_cfg, options);
     solution.solver_id = s.SolverDbId();
     return solution;
 }
 
+template <class Solver, class Context, class Problem>
+InvokerFactory GetInvokeFactory(Solver s,
+                                const Context& context,
+                                const Problem& problem,
+                                const std::string& perf_cfg)
+{
+    static_assert(sizeof(Solver) == sizeof(SolverBase), "Solver must be stateless");
+    static_assert(std::is_base_of<SolverBase, Solver>{}, "Not derived class of SolverBase");
+    // TODO: This assumes all solutions are ConvSolution
+    return GetInvokeFactoryImpl(rank<1>{}, s, context, problem, perf_cfg);
+}
+
 template <class... Solvers>
 struct SolverContainer
 {
@@ -203,6 +263,13 @@ struct SolverContainer
             Solvers{}...);
     }
 
+    ///\todo: remove when AnySolver would be able to work with non-conv solvers
+    template <class Functor>
+    void Foreach(Functor&& receiver)
+    {
+        miopen::each_args([&](auto solver) { receiver(solver); }, Solvers{}...);
+    }
+
     // Search for all applicable solutions among many solvers
     template <class Context, class Problem, class Db, class Solution = miopen::solver::ConvSolution>
     std::vector<Solution>
@@ -266,8 +333,10 @@ struct SolverContainer
     std::vector<Solution>
     SearchForSolutions(const ExecutionContext& ctx,
                        const Problem& problem,
-                       std::size_t limit = std::numeric_limits<std::size_t>::max()) const
+                       std::size_t limit = std::numeric_limits<std::size_t>::max(),
+                       const AnyInvokeParams& invoke_params = {}) const
     {
+        auto db_container = std::optional<PerformanceDb>{};
         std::vector<Solution> ss;
         std::size_t count    = 0;
         const auto find_only = GetEnvFindOnlySolver();
@@ -290,12 +359,29 @@ struct SolverContainer
                 }
                 else
                 {
-                    auto s      = solver.GetSolution(ctx, problem);
-                    s.solver_id = solver.SolverDbId();
+                    auto db = [&]() -> PerformanceDb& {
+                        constexpr auto db_getter =
+                            []([[maybe_unused]] const ExecutionContext& ctx,
+                               [[maybe_unused]] const auto& problem) -> PerformanceDb {
+                            if constexpr(IsTunable<decltype(solver)>())
+                                return GetDb(ctx, problem);
+                            else
+                                MIOPEN_THROW(miopenStatusInternalError);
+                        };
+
+                        if(!db_container)
+                            db_container.emplace(std::move(db_getter(ctx, problem)));
+
+                        return *db_container;
+                    };
+
+                    auto s =
+                        FindSolution(solver, ctx, problem, db, invoke_params, "", std::nullopt);
+
                     if(s.Succeeded())
                     {
                         ++count;
-                        ss.push_back(s);
+                        ss.emplace_back(std::move(s));
                         MIOPEN_LOG_I2(solver.SolverDbId() << ": Success.");
                     }
                     else
@@ -309,25 +395,25 @@ struct SolverContainer
     }
 
     template <class Context, class Problem>
-    std::vector<std::pair<std::string, size_t>>
-    GetWorkspaceSizes(const Context& ctx,
-                      const Problem& problem,
-                      std::size_t limit = std::numeric_limits<std::size_t>::max()) const
+    std::vector<std::pair<std::string, size_t>> GetWorkspaceSizes(
+        const Context& ctx, const Problem& problem, const bool simple_primitive = false) const
     {
         std::vector<std::pair<std::string, size_t>> res;
         const auto find_only = GetEnvFindOnlySolver();
-        std::size_t count    = 0;
         miopen::each_args(
             [&](auto solver) {
-                if(count >= limit)
-                    return;
-
                 if(find_only &&
                    (std::find(find_only->begin(), find_only->end(), Id{solver.SolverDbId()}) ==
                     find_only->end()))
                 { // Do nothing (and keep silence for the sake of Tuna), just skip.
                 }
-                else if(!solver.MayNeedWorkspace())
+                // The following optimization is required to avoid checks
+                // for solvers that have slow IsApplicable() and do not
+                // require workspace (like MLIR convolutions). However we
+                // do not want to use it for simple primitives, for example,
+                // the ones that ExecutePrimitive() which uses the first applicable
+                // solver:
+                else if(!simple_primitive && !solver.MayNeedWorkspace())
                 {
                     MIOPEN_LOG_I2(solver.SolverDbId() << ": Skipped (no workspace required)");
                 }
@@ -343,7 +429,6 @@ struct SolverContainer
                 }
                 else
                 {
-                    ++count;
                     auto sz = solver.GetWorkspaceSize(ctx, problem);
                     res.push_back(std::make_pair(solver.SolverDbId(), sz));
                     MIOPEN_LOG_I2(solver.SolverDbId() << ": " << sz);
@@ -389,21 +474,21 @@ struct SolverContainer
     }
 
     template <class Problem>
-    void ExecutePrimitive(Handle& handle,
+    void ExecutePrimitive(const ExecutionContext& ctx,
                           const Problem& problem,
                           const AlgorithmName& algo,
                           const AnyInvokeParams& invoke_params) const
     {
         const auto network_config = problem.MakeNetworkConfig();
 
-        if(const auto existingInvoker = handle.GetInvoker(network_config, boost::none, algo))
+        if(const auto existingInvoker =
+               ctx.GetStream().GetInvoker(network_config, std::nullopt, algo))
         {
-            (*existingInvoker)(handle, invoke_params);
+            (*existingInvoker)(ctx.GetStream(), invoke_params);
             return;
         }
 
-        auto ctx        = ExecutionContext{&handle};
-        const auto slns = SearchForSolutions(ctx, problem, 1);
+        const auto slns = SearchForSolutions(ctx, problem, 1, invoke_params);
 
         if(slns.empty())
             MIOPEN_THROW(miopenStatusNotImplemented, "No solver found.");
@@ -411,9 +496,19 @@ struct SolverContainer
         const auto& sln = slns.front();
         if(!sln.invoker_factory)
             MIOPEN_THROW(miopenStatusInternalError, "Invoker missing in solver " + sln.solver_id);
-        const auto invoker = handle.PrepareInvoker(*sln.invoker_factory, sln.construction_params);
-        handle.RegisterInvoker(invoker, network_config, sln.solver_id, algo);
-        invoker(handle, invoke_params);
+        const auto invoker =
+            ctx.GetStream().PrepareInvoker(*sln.invoker_factory, sln.construction_params);
+        ctx.GetStream().RegisterInvoker(invoker, network_config, sln.solver_id, algo);
+        invoker(ctx.GetStream(), invoke_params);
+    }
+
+    template <class Problem>
+    void ExecutePrimitive(Handle& handle,
+                          const Problem& problem,
+                          const AlgorithmName& algo,
+                          const AnyInvokeParams& invoke_params) const
+    {
+        return ExecutePrimitive(&handle, problem, algo, invoke_params);
     }
 };
 
