@@ -39,11 +39,8 @@
 #include <miopen/ocldeviceinfo.hpp>
 #include <miopen/timer.hpp>
 
-#if MIOPEN_USE_MIOPENGEMM
-#include <miopen/gemm_geometry.hpp>
-#endif
-
-#include <boost/filesystem.hpp>
+#include <miopen/filesystem.hpp>
+#include <boost/filesystem/operations.hpp>
 
 #include <string>
 
@@ -91,9 +88,6 @@ struct HandleImpl
     {
         std::string name = miopen::GetDeviceInfo<CL_DEVICE_NAME>(device);
         MIOPEN_LOG_NQI("Raw device name: " << name);
-#if WORKAROUND_MLOPEN_ISSUE_1711
-        WorkaroundIssue1711(name);
-#endif
         return name;
     }
 
@@ -195,7 +189,7 @@ static bool PrintOpenCLDeprecateMsg()
     MIOPEN_LOG_W(
         "please port your application to the better supported and functional HIP backend. ");
     MIOPEN_LOG_W("If you have any questions, please reach out to the MIOpen developers at ");
-    MIOPEN_LOG_W("https://github.com/ROCmSoftwarePlatform/MIOpen");
+    MIOPEN_LOG_W("https://github.com/ROCm/MIOpen");
     return true;
 }
 
@@ -324,10 +318,8 @@ KernelInvoke Handle::AddKernel(const std::string& algorithm,
                                const std::vector<size_t>& vgd,
                                const std::string& params,
                                std::size_t cache_index,
-                               bool is_kernel_str,
                                const std::string& kernel_src) const
 {
-
     auto obj = this->impl->cache.AddKernel(*this,
                                            algorithm,
                                            network_config,
@@ -337,14 +329,16 @@ KernelInvoke Handle::AddKernel(const std::string& algorithm,
                                            vgd,
                                            params,
                                            cache_index,
-                                           is_kernel_str,
                                            kernel_src);
     return this->Run(obj);
 }
 
 Invoker Handle::PrepareInvoker(const InvokerFactory& factory,
-                               const std::vector<solver::KernelInfo>& kernels) const
+                               const std::vector<solver::KernelInfo>& kernels,
+                               std::vector<Program>* programs_out) const
 {
+    std::ignore = programs_out;
+
     std::vector<Kernel> built;
     for(auto& k : kernels)
     {
@@ -375,8 +369,11 @@ const std::vector<Kernel>& Handle::GetKernelsImpl(const std::string& algorithm,
     return this->impl->cache.GetKernels(algorithm, network_config);
 }
 
-KernelInvoke Handle::Run(Kernel k) const
+KernelInvoke Handle::Run(Kernel k, bool coop_launch) const
 {
+    if(coop_launch)
+        MIOPEN_THROW(miopenStatusInternalError);
+
     auto q = this->GetStream();
     if(this->impl->enable_profiling || MIOPEN_GPU_SYNC)
     {
@@ -393,14 +390,14 @@ KernelInvoke Handle::Run(Kernel k) const
 
 Program Handle::LoadProgram(const std::string& program_name,
                             std::string params,
-                            bool is_kernel_str,
-                            const std::string& kernel_src) const
+                            const std::string& kernel_src,
+                            bool force_attach_binary) const
 {
-    auto hsaco = miopen::LoadBinary(this->GetTargetProperties(),
-                                    this->GetMaxComputeUnits(),
-                                    program_name,
-                                    params,
-                                    is_kernel_str);
+    // Binary serialization is not supported on OpenCL anyway
+    std::ignore = force_attach_binary;
+
+    auto hsaco = miopen::LoadBinary(
+        this->GetTargetProperties(), this->GetMaxComputeUnits(), program_name, params);
     if(hsaco.empty())
     {
         CompileTimer ct;
@@ -409,25 +406,19 @@ Program Handle::LoadProgram(const std::string& program_name,
                                      this->GetTargetProperties(),
                                      program_name,
                                      params,
-                                     is_kernel_str,
                                      kernel_src);
-        ct.Log("Kernel", is_kernel_str ? std::string() : program_name);
+        ct.Log("Kernel", program_name);
 
 // Save to cache
 #if MIOPEN_ENABLE_SQLITE_KERN_CACHE
         std::string binary;
         miopen::GetProgramBinary(p, binary);
-        miopen::SaveBinary(binary,
-                           this->GetTargetProperties(),
-                           this->GetMaxComputeUnits(),
-                           program_name,
-                           params,
-                           is_kernel_str);
-#else
-        auto path = miopen::GetCachePath(false) / boost::filesystem::unique_path();
-        miopen::SaveProgramBinary(p, path.string());
         miopen::SaveBinary(
-            path.string(), this->GetTargetProperties(), program_name, params, is_kernel_str);
+            binary, this->GetTargetProperties(), this->GetMaxComputeUnits(), program_name, params);
+#else
+        auto path = miopen::GetCachePath(false) / boost::filesystem::unique_path().string();
+        miopen::SaveProgramBinary(p, path.string());
+        miopen::SaveBinary(path, this->GetTargetProperties(), program_name, params);
 #endif
         return p;
     }
@@ -499,6 +490,8 @@ std::size_t Handle::GetMaxMemoryAllocSize()
     return m_MaxMemoryAllocSizeCached;
 }
 
+bool Handle::CooperativeLaunchSupported() const { return false; }
+
 std::size_t Handle::GetMaxComputeUnits() const
 {
     return miopen::GetDeviceInfo<CL_DEVICE_MAX_COMPUTE_UNITS>(miopen::GetDevice(this->GetStream()));
