@@ -80,7 +80,7 @@ miopenStatus_t PoolingDescriptor::Forward(Handle& handle,
         }
     }
 
-    int pool_dim = xDesc.GetSize();
+    unsigned pool_dim = xDesc.GetNumDims();
     if(pool_dim != 4 && pool_dim != 5)
     {
         MIOPEN_THROW("Unsupported pooling dimension");
@@ -171,7 +171,7 @@ miopenStatus_t PoolingDescriptor::Backward(Handle& handle,
     assert(yDesc.GetElementSize() == dyDesc.GetElementSize() &&
            xDesc.GetElementSize() == dxDesc.GetElementSize());
 
-    int pool_dim = dyDesc.GetSize();
+    unsigned pool_dim = dyDesc.GetNumDims();
     if(pool_dim != 4 && pool_dim != 5)
     {
         MIOPEN_THROW("Unsupported pooling dimension");
