@@ -977,7 +977,7 @@ void OpTensorOther(const Handle& handle,
     const bool case_2d = bsize == 2;
     const bool case_5d = bsize == 5;
 
-    const bool use_hip = case_1d;
+    const bool use_hip = case_1d || case_2d;
 
     // first_not_one is incorrect if btensor size equal to 1
     auto first_not_one = std::find_if(blens.rbegin(), blens.rend(), [](int i) { return i != 1; });
@@ -1005,22 +1005,28 @@ void OpTensorOther(const Handle& handle,
 
     int num_wg_orig = num_wg;
     int max_num_wg  = 4096;
-    num_wg          = num_wg > max_num_wg ? max_num_wg : num_wg;
 
     size_t local_threads = 256;
 
-    std::string program_name = use_hip ? "MIOpenTensorKernelsHip.cpp" : "MIOpenTensorKernels.cl";
+    if(case_2d)
+        local_threads = 32;
+
+    if(case_1d)
+        num_wg = std::clamp(clens[0] / local_threads, size_t(1), size_t(max_num_wg));
+    if(case_2d)
+        num_wg = std::clamp((clens[0] * clens[1]) / local_threads, size_t(1), size_t(max_num_wg));
+    num_wg = num_wg > max_num_wg ? max_num_wg : num_wg;
 
     const std::vector<size_t> vld{local_threads, 1, 1};
 
     // Special case for adding tensors in place
     size_t global_threads;
-    global_threads =
-        (case_1d ? std::clamp(clens[0] / local_threads, size_t(1), size_t(max_num_wg)) : num_wg) *
-        local_threads;
+    global_threads = num_wg * local_threads;
 
     const std::vector<size_t> vgd{global_threads, 1, 1};
 
+    std::string program_name = use_hip ? "MIOpenTensorKernelsHip.cpp" : "MIOpenTensorKernels.cl";
+
     std::string network_config{};
     network_config += std::to_string(bTensorDesc.GetType()) + "-" +
                       std::to_string(aTensorDesc.GetType()) + "-" + std::to_string(tensorOp) + "-" +
@@ -1081,22 +1087,24 @@ void OpTensorOther(const Handle& handle,
             {
                 auto kernel = kernels.front();
                 kernel(ATensor,
-                       static_cast<int>(astrides[0]),
                        BTensor,
-                       static_cast<int>(blens[1]),
-                       static_cast<int>(bstrides[0]),
                        CTensor,
-                       static_cast<int>(clens[1]),
-                       static_cast<int>(cstrides[0]),
+                       static_cast<long>(Aoffset),
+                       static_cast<long>(Boffset),
+                       static_cast<long>(Coffset),
+                       static_cast<uint32_t>(blens[1] == 1 ? clens[1] : blens[1]),
+                       static_cast<uint32_t>(clens[1]),
+                       static_cast<uint32_t>(astrides[0]),
+                       static_cast<uint32_t>(astrides[1]),
+                       static_cast<uint32_t>(blens[0] == 1 ? 0 : bstrides[0]),
+                       static_cast<uint32_t>(blens[1] == 1 ? 0 : bstrides[1]),
+                       static_cast<uint32_t>(cstrides[0]),
+                       static_cast<uint32_t>(cstrides[1]),
                        miopen_alpha0,
                        miopen_alpha1,
                        miopen_beta,
-                       bitmap,
-                       work_per_wg,
-                       static_cast<int64_t>(Aoffset),
-                       static_cast<int64_t>(Boffset),
-                       static_cast<int64_t>(Coffset),
-                       static_cast<int>(num_wg_orig));
+                       static_cast<uint32_t>(clens[0]),
+                       !float_equal(miopen_beta, 0.0));
                 return;
             }
         }
@@ -1194,22 +1202,24 @@ void OpTensorOther(const Handle& handle,
                              vld,
                              vgd,
                              parms)(ATensor,
-                                    static_cast<int>(astrides[0]),
                                     BTensor,
-                                    static_cast<int>(blens[1]),
-                                    static_cast<int>(bstrides[0]),
                                     CTensor,
-                                    static_cast<int>(clens[1]),
-                                    static_cast<int>(cstrides[0]),
+                                    static_cast<long>(Aoffset),
+                                    static_cast<long>(Boffset),
+                                    static_cast<long>(Coffset),
+                                    static_cast<uint32_t>(blens[1] == 1 ? clens[1] : blens[1]),
+                                    static_cast<uint32_t>(clens[1]),
+                                    static_cast<uint32_t>(astrides[0]),
+                                    static_cast<uint32_t>(astrides[1]),
+                                    static_cast<uint32_t>(blens[0] == 1 ? 0 : bstrides[0]),
+                                    static_cast<uint32_t>(blens[1] == 1 ? 0 : bstrides[1]),
+                                    static_cast<uint32_t>(cstrides[0]),
+                                    static_cast<uint32_t>(cstrides[1]),
                                     miopen_alpha0,
                                     miopen_alpha1,
                                     miopen_beta,
-                                    bitmap,
-                                    work_per_wg,
-                                    static_cast<int64_t>(Aoffset),
-                                    static_cast<int64_t>(Boffset),
-                                    static_cast<int64_t>(Coffset),
-                                    static_cast<int>(num_wg_orig));
+                                    static_cast<uint32_t>(clens[0]),
+                                    !float_equal(miopen_beta, 0.0));
         }
         else if(case_1d)
         {
@@ -1430,15 +1440,14 @@ void SetTensor(const Handle& handle,
     const TensorDescriptor yDesc_flat = GetFlattenedTensorDescriptor(yDesc);
 
 #ifndef NDEBUG
-    if(yDesc.GetSize() != yDesc_flat.GetSize())
+    if(yDesc.GetNumDims() != yDesc_flat.GetNumDims())
     {
-        MIOPEN_LOG_I2(__func__ << std::endl
-                               << "real descriptor: " << yDesc << std::endl
-                               << "flat descriptor: " << yDesc_flat << std::endl);
+        MIOPEN_LOG_I2("real descriptor: " << yDesc);
+        MIOPEN_LOG_I2("flat descriptor: " << yDesc_flat);
     }
 #endif
 
-    const std::size_t yDim_flat = yDesc_flat.GetSize();
+    const std::size_t yDim_flat = yDesc_flat.GetNumDims();
 
     assert(yDim_flat > 0 && yDim_flat <= 5);
 
@@ -1584,15 +1593,14 @@ void ScaleTensor(const Handle& handle,
     const TensorDescriptor yDesc_flat = GetFlattenedTensorDescriptor(yDesc);
 
 #ifndef NDEBUG
-    if(yDesc.GetSize() != yDesc_flat.GetSize())
+    if(yDesc.GetNumDims() != yDesc_flat.GetNumDims())
     {
-        MIOPEN_LOG_I2(__func__ << std::endl
-                               << "real descriptor: " << yDesc << std::endl
-                               << "flat descriptor: " << yDesc_flat << std::endl);
+        MIOPEN_LOG_I2("real descriptor: " << yDesc);
+        MIOPEN_LOG_I2("flat descriptor: " << yDesc_flat);
     }
 #endif
 
-    const std::size_t yDim_flat = yDesc_flat.GetSize();
+    const std::size_t yDim_flat = yDesc_flat.GetNumDims();
 
     assert(yDim_flat > 0 && yDim_flat <= 5);
 
@@ -1763,17 +1771,16 @@ void CopyTensor(const Handle& handle,
     const TensorDescriptor& dstDesc_flat = std::get<1>(flat_descriptors);
 
 #ifndef NDEBUG
-    if(srcDesc.GetSize() != srcDesc_flat.GetSize())
+    if(srcDesc.GetNumDims() != srcDesc_flat.GetNumDims())
     {
-        MIOPEN_LOG_I2(__func__ << std::endl
-                               << "src real descriptor: " << srcDesc << std::endl
-                               << "src flat descriptor: " << srcDesc_flat << std::endl
-                               << "dst real descriptor: " << dstDesc << std::endl
-                               << "dst flat descriptor: " << dstDesc_flat << std::endl);
+        MIOPEN_LOG_I2("src real descriptor: " << srcDesc);
+        MIOPEN_LOG_I2("src flat descriptor: " << srcDesc_flat);
+        MIOPEN_LOG_I2("dst real descriptor: " << dstDesc);
+        MIOPEN_LOG_I2("dst flat descriptor: " << dstDesc_flat);
     }
 #endif
 
-    std::size_t srcDim_flat = srcDesc_flat.GetSize();
+    std::size_t srcDim_flat = srcDesc_flat.GetNumDims();
 
     if(srcDim_flat < 1 || srcDim_flat > 5)
     {
@@ -1944,6 +1951,8 @@ std::string GetCastTensorBuildOptionFromType(const std::string& buildOption, mio
     case miopenDouble:
         // TODO
         MIOPEN_THROW(miopenStatusBadParm, "miopenDouble data type not supported in cast tensor.");
+    case miopenInt64:
+        MIOPEN_THROW(miopenStatusBadParm, "miopenInt64 data type not supported in cast tensor.");
     default: MIOPEN_THROW(miopenStatusBadParm, "Invalid data type in cast tensor desc.");
     }
 }
@@ -1973,17 +1982,16 @@ void CastTensor(const Handle& handle,
     const TensorDescriptor& dstDesc_flat = std::get<1>(flat_descriptors);
 
 #ifndef NDEBUG
-    if(srcDesc.GetSize() != srcDesc_flat.GetSize())
+    if(srcDesc.GetNumDims() != srcDesc_flat.GetNumDims())
     {
-        MIOPEN_LOG_I2(__func__ << std::endl
-                               << "src real descriptor: " << srcDesc << std::endl
-                               << "src flat descriptor: " << srcDesc_flat << std::endl
-                               << "dst real descriptor: " << dstDesc << std::endl
-                               << "dst flat descriptor: " << dstDesc_flat << std::endl);
+        MIOPEN_LOG_I2("src real descriptor: " << srcDesc);
+        MIOPEN_LOG_I2("src flat descriptor: " << srcDesc_flat);
+        MIOPEN_LOG_I2("dst real descriptor: " << dstDesc);
+        MIOPEN_LOG_I2("dst flat descriptor: " << dstDesc_flat);
     }
 #endif
 
-    std::size_t srcDim_flat = srcDesc_flat.GetSize();
+    std::size_t srcDim_flat = srcDesc_flat.GetNumDims();
 
     if(srcDim_flat < 1 || srcDim_flat > 5)
     {
@@ -2189,6 +2197,9 @@ void TransformTensor(const Handle& handle,
         MIOPEN_THROW("Tensor x and y batch sizes do not match");
     }
 
+    const auto is_alpha_one = float_equal(*(static_cast<const float*>(alpha)), 1);
+    const auto is_beta_zero = float_equal(*(static_cast<const float*>(beta)), 0);
+
     if(xDesc.GetType() == miopenInt8 && yDesc.GetType() == miopenInt8 && x_len.size() >= 3)
     {
         if(x_len[1] <= y_len[1])
@@ -2223,8 +2234,7 @@ void TransformTensor(const Handle& handle,
             size_t x_offset = i * x_batch_sz;
             size_t y_offset = i * y_batch_sz;
 
-            if(float_equal(*(static_cast<const float*>(alpha)), 1) &&
-               float_equal(*(static_cast<const float*>(beta)), 0))
+            if(is_alpha_one && is_beta_zero)
             {
                 CopyTensor(handle,
                            ((x_len[1] <= y_len[1]) ? x_batch_desc : y_batch_desc),
@@ -2236,7 +2246,8 @@ void TransformTensor(const Handle& handle,
             }
             else
             {
-                // TODO: support y=alpha*x+beta*y
+                MIOPEN_THROW(miopenStatusNotImplemented,
+                             "y=alpha*x+beta*y is not supported for int8 yet");
             }
         }
     }
@@ -2256,23 +2267,19 @@ void TransformTensor(const Handle& handle,
         const TensorDescriptor& xDesc_flat = std::get<0>(flat_descriptors);
         const TensorDescriptor& yDesc_flat = std::get<1>(flat_descriptors);
 
-#ifndef NDEBUG
-        if(xDesc.GetSize() != xDesc_flat.GetSize())
+        if(xDesc.GetNumDims() != xDesc_flat.GetNumDims())
         {
-            MIOPEN_LOG_I2(__func__ << std::endl
-                                   << "real descriptor: " << xDesc << std::endl
-                                   << "flat descriptor: " << xDesc_flat << std::endl);
+            MIOPEN_LOG_I2("x real descriptor: " << xDesc);
+            MIOPEN_LOG_I2("x flat descriptor: " << xDesc_flat);
         }
 
-        if(yDesc.GetSize() != yDesc_flat.GetSize())
+        if(yDesc.GetNumDims() != yDesc_flat.GetNumDims())
         {
-            MIOPEN_LOG_I2(__func__ << std::endl
-                                   << "real descriptor: " << yDesc << std::endl
-                                   << "flat descriptor: " << yDesc_flat << std::endl);
+            MIOPEN_LOG_I2("y real descriptor: " << yDesc);
+            MIOPEN_LOG_I2("y flat descriptor: " << yDesc_flat);
         }
-#endif
 
-        const std::size_t yDim_flat = yDesc_flat.GetSize();
+        const std::size_t yDim_flat = yDesc_flat.GetNumDims();
 
         assert(yDim_flat > 0 && yDim_flat <= 5);
 
@@ -2312,6 +2319,11 @@ void TransformTensor(const Handle& handle,
             network_config += "x" + std::to_string(len);
         }
 
+        if(is_beta_zero)
+            network_config += "xBETA_IS_ZERO";
+        if(is_alpha_one)
+            network_config += "xALPHA_IS_ONE";
+
         auto&& kernels = handle.GetKernels(kernel_name, network_config);
 
         KernelInvoke kernel;
@@ -2333,8 +2345,10 @@ void TransformTensor(const Handle& handle,
 
             std::size_t wld = 256 < wgd ? 256 : wgd;
 
-            std::string parms = "-DSUBTENSOR_OP_WITH_SCALAR=SUBTENSOR_OP_WITH_SCALAR_MAD" +
-                                GetDataTypeKernelParams(dataTypey);
+            std::string parms =
+                GetDataTypeKernelParams(dataTypey)                                           //
+                + " -DMIOPEN_BETA_IS_ZERO=" + std::to_string(static_cast<int>(is_beta_zero)) //
+                + " -DMIOPEN_ALPHA_IS_ONE=" + std::to_string(static_cast<int>(is_alpha_one));
 
             for(int i = 0; i < yDim_flat; ++i)
             {
