@@ -327,6 +327,9 @@ struct verify_forward_train_3d_bn_spatial
                                          miopen::BuildReshaped4DTensorDescriptor(out.desc),
                                          out_dev.get(),
                                          miopen::BuildReshaped4DTensorDescriptor(scale.desc),
+                                         miopen::BuildReshaped4DTensorDescriptor(shift.desc),
+                                         miopen::BuildReshaped4DTensorDescriptor(shift.desc),
+                                         miopen::BuildReshaped4DTensorDescriptor(shift.desc),
                                          scale_dev.get(),
                                          shift_dev.get(),
                                          expAvgFactor,
@@ -516,6 +519,9 @@ struct verify_forward_infer_3d_bn_spatial_recalc
                                           miopen::BuildReshaped4DTensorDescriptor(out.desc),
                                           out_dev.get(),
                                           miopen::BuildReshaped4DTensorDescriptor(scale.desc),
+                                          miopen::BuildReshaped4DTensorDescriptor(shift.desc),
+                                          miopen::BuildReshaped4DTensorDescriptor(shift.desc),
+                                          miopen::BuildReshaped4DTensorDescriptor(shift.desc),
                                           scale_dev.get(),
                                           shift_dev.get(),
                                           nullptr,
@@ -632,6 +638,9 @@ struct verify_forward_infer_3d_bn_spatial_use_est
                                           miopen::BuildReshaped4DTensorDescriptor(out.desc),
                                           out_dev.get(),
                                           miopen::BuildReshaped4DTensorDescriptor(scale.desc),
+                                          miopen::BuildReshaped4DTensorDescriptor(shift.desc),
+                                          miopen::BuildReshaped4DTensorDescriptor(shift.desc),
+                                          miopen::BuildReshaped4DTensorDescriptor(shift.desc),
                                           scale_dev.get(),
                                           shift_dev.get(),
                                           estMean_dev.get(),
@@ -913,6 +922,9 @@ struct verify_backward_3d_bn_spatial_recalc
                                   miopen::BuildReshaped4DTensorDescriptor(dx_out.desc),
                                   dx_out_dev.get(),
                                   miopen::BuildReshaped4DTensorDescriptor(scale.desc),
+                                  miopen::BuildReshaped4DTensorDescriptor(dshift.desc),
+                                  miopen::BuildReshaped4DTensorDescriptor(dshift.desc),
+                                  miopen::BuildReshaped4DTensorDescriptor(dshift.desc),
                                   scale_dev.get(),
                                   dscale_dev.get(),
                                   dshift_dev.get(),
@@ -1138,6 +1150,9 @@ struct verify_backward_3d_bn_spatial_use_saved
                                   miopen::BuildReshaped4DTensorDescriptor(dx_out.desc),
                                   dx_out_dev.get(),
                                   miopen::BuildReshaped4DTensorDescriptor(scale.desc),
+                                  miopen::BuildReshaped4DTensorDescriptor(dshift.desc),
+                                  miopen::BuildReshaped4DTensorDescriptor(dshift.desc),
+                                  miopen::BuildReshaped4DTensorDescriptor(dshift.desc),
                                   scale_dev.get(),
                                   dscale_dev.get(),
                                   dshift_dev.get(),
